<?php

/*
  
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * <p>Renders HTML block which allows to call DPD courier to pick up shipment.</p>
 * <p>Template is located at <b>eabi_dpdee/order/courier.phtml</b> in adminhtml theme folder</p>
 * <p>Information that can be sent to courier:</p>
 * <ul>
     <li>Merchant can pick suitable date from available list when courier should come</li>
     <li>Merchant can pick suitable time-range from available list for the selected date</li>
     <li>Merchant can specify how many envelopes, parcels, pallets should be picked up</li>
     <li>Merchant can leave comment for the courier</li>
     <li>By default one checked order equals one parcel</li>
 </ul>
 *
 * @author Matis
 */
class Eabi_DpdEE_Block_Order_Courier extends Mage_Core_Block_Template {
    protected $_dateFormat = 'yyyy-MM-dd';
    protected $_timeFormat = 'HHmm';
    protected $_timeFormatNice = 'HH:mm';
    
    protected $_availableDates;
    protected $_apiResult;
    
    public function _construct() {
        $this->setTemplate('eabi_dpdee/order/courier.phtml');
        parent::_construct();
    }
    
    /**
     * <p>Only one block is allowed per page</p>
     * @return string
     */
    public function getHtmlId() {
        return 'eabi_carrier_order_courier_box';
    }
    




    /**
     * <p>Fetches list of available pickup dates from DPD server and returns it as HTML select string</p>
     * @return string
     */
    public function getAvailableDates() {
        if ($this->_availableDates) {
            return $this->_availableDates;
        }
        $showOnlyDates = false;
        $datesResult = $this->_getDpdHelper()->getApi($this->getStoreId(), $this->getCode())->getCourierCollectionTimes();
        $this->_apiResult = $datesResult;
        $dateObj = null;

        $resultString = '';
//        $resultString .= '<pre>'.htmlspecialchars(print_r($datesResult, true)).'</pre>';
            $resultString .= '<select name="Po_Date" id="Po_date" onchange="updateDpdEeTimes(this);">';
            $resultString .= '<option value="-">'.$this->_getDpdHelper()->__(' - select pickup date - ').'</option>';

        foreach ($datesResult['Po_date'] as $dateString) {
            if (is_null($dateObj)) {
                $dateObj = new Zend_Date($dateString, $this->_dateFormat);
            }
            $dateObj->set($dateString, $this->_dateFormat);
            if ($dateObj->isToday() && !$showOnlyDates) {
                //today
                $resultString .= $this->_getDateInputLabel($this->_getDpdHelper()->__('Today'), $dateString);
            } else if ($dateObj->isTomorrow() && !$showOnlyDates) {
                //tomorrow
                $resultString .= $this->_getDateInputLabel($this->_getDpdHelper()->__('Tomorrow'), $dateString);
            } else {
                //other dates
                $resultString .= $this->_getDateInputLabel($dateString, $dateString);
            }
        }
        $resultString .= '</select>';

        //do availabletimes as well
        $this->_availableDates = $resultString;
        return $this->_availableDates;
    }
    
    
    /**
     * <p>Fetches available pickup times from DPD server and returns it as json encoded object</p>
     * <p>Format:</p>
     * <pre>
     *  array(
     *      '2013-12-24' => 'html select menu with available time ranges',
     *      '2013-12-25' => 'html select menu with available time ranges',
     *      ....
     * );
     * </pre>
     * @return string
     */
    public function getAvailabeTimes() {
        if (!$this->_apiResult) {
            $this->getAvailableDates();
        }
        $availableTimes = array();
        $timeDateObject = null;
        //array key is date, content is html with select menu
        $availableTimes["-"] = $this->_getDpdHelper()->__('Select pickup date first');
        foreach ($this->_apiResult['Po_date'] as $key => $dateString) {
            $timeFrom = $this->_normalizeTime($this->_apiResult['Po_time_from'][$key]);
            $timeTo = $this->_normalizeTime($this->_apiResult['Po_time_til'][$key]);
            $timeWindow = $this->_normalizeTime($this->_apiResult['Minimal_time_window'][$key]);
            $availableTimes[$dateString] = $this->_getTimeSelectMenu($timeFrom, $timeTo, $timeWindow);
        }
        return json_encode($availableTimes);
    }
    
    /**
     * <p>Normalizes DPD times to Zend_Date objects.</p>
     * <p>For example 930 should be displayed as 09:30 and 9 should be displayed as 09:00</p>
     * @param string $input
     * @return Zend_Date
     */
    protected function _normalizeTime($input) {
        if (strlen($input) === 1) {
            $input = '0'.$input .'00';
        }
        if (strlen($input) === 2) {
            $input = $input .'00';
        }
        if (strlen($input) === 3) {
            $input = '0'.$input;
        }
        $date = new Zend_Date($input, $this->_timeFormat);
        $date->setTimezone('Etc/GMT+0');
        return $date;
    }


    /**
     * <p>Takes earliest possible time, latest possible time, minimum allowed timewindow and renders it into one select menu with pickup time ranges.</p>
     * <p>All arguments have to be within same date.</p>
     * <p>Pickup time ranges are always displayed within narrowest possible timewindow.</p>
     * @param Zend_Date $timeFrom earliest time possible
     * @param Zend_Date $timeTo latest time possible
     * @param Zend_Date $timeWindow minimal allowed time between start and end time.
     * @return string HTML select menu
     */
    protected function _getTimeSelectMenu(Zend_Date $timeFrom, Zend_Date $timeTo, Zend_Date $timeWindow) {
        $position = $timeFrom->get('H');
        $result = '<select name="Po_Time" id="Po_Time">';
        do {
            $endTime = clone $timeFrom;
            $endTime->add($timeWindow);
            if ($timeTo->isEarlier($endTime)) {
                $timeDisplay = htmlspecialchars(ltrim($timeFrom->get($this->_timeFormat), '0') . '-' . ltrim($timeTo->get($this->_timeFormat), '0'));
                $timeDisplayNice = htmlspecialchars($timeFrom->get($this->_timeFormatNice) . '-' . $timeTo->get($this->_timeFormatNice));
            } else {
                $timeDisplay = htmlspecialchars(ltrim($timeFrom->get($this->_timeFormat), '0') . '-' . ltrim($endTime->get($this->_timeFormat), '0'));
                $timeDisplayNice = htmlspecialchars($timeFrom->get($this->_timeFormatNice) . '-' . $endTime->get($this->_timeFormatNice));
            }
            $result .= '<option value="' . $timeDisplay . '">' . $timeDisplayNice . '</option>';
            $timeFrom->setTimestamp($timeFrom->getTimestamp() + 60 * 60);
        } while ($timeFrom->get(Zend_Date::HOUR_SHORT) + $timeWindow->get(Zend_Date::HOUR_SHORT) <= $timeTo->get(Zend_Date::HOUR_SHORT));
        $result .= '</select>';
        return $result;
    }

    protected function _getDateInputLabel($label, $value) {
        return '<option value="'.  htmlspecialchars($value).'" >'.  htmlspecialchars($label).'</option>';
    }
    
    /**
     * <p>Returns shipping method code that this courier HTML block is used by.</p>
     * @return string
     */
    public function getCode() {
        if (!$this->getData('code')) {
            return 'eabidpdee';
        }
        return $this->getData('code');
    }
    
    /**
     * <p>Returns default envelope quantity</p>
     * @return string
     */
    public function getEnvelopeQty() {
        return '0';
    }
    
    
    /**
     * <p>Returns default parcel quantity</p>
     * @return string
     */
    public function getParcelQty() {
        return '1';
        
    }
    
    
    /**
     * <p>Returns default pallet quantity</p>
     * @return string
     */
    public function getPalletQty() {
        return '0';
        
    }
    
    /**
     * 
     * @return Eabi_DpdEE_Helper_Data
     */
    protected function _getDpdHelper() {
        return Mage::helper('eabi_dpdee');
    }
    
    
}
