<?php

/*

 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 
 
 */

/**
 * <p>Helper class for DPD shipping method related actions</p>
 *
 * @author Matis Halmann
 */
class Eabi_DpdEE_Helper_Data extends Mage_Core_Helper_Abstract {
    protected $_apis = array();

    /**
     * <p>Gets cached DPD API instance for specified Magento store id and shipping method code.</p>
     * @param string $storeId store id to fetch the api for
     * @param string $code shipping method code to fetch the api for
     * @return Eabi_DpdEE_Model_Api
     */
    public function getApi($storeId = null, $code = 'eabidpdee') {
        if ($storeId === null) {
            $storeId = Mage::app()->getStore($storeId)->getId();
        }
        if (isset($this->_apis[$code]) && isset($this->_apis[$code][$storeId])) {
            return $this->_apis[$code][$storeId];
        }
        if (!isset($this->_apis[$code])) {
            $this->_apis[$code] = array();
        }
        $api = Mage::getModel('eabi_dpdee/api');
        $api->setStore($storeId);
        $api->setCode($code);
        $this->_apis[$code][$storeId] = $api;
        return $this->_apis[$code][$storeId];
    }
    
    
    /**
     * Returns true if shipping method used in the order belongs to DPD carrier
     * @param Mage_Sales_Model_Order $order
     */
    public function isShippingMethodApplicable(Mage_Sales_Model_Order $order) {
        return strpos($order->getShippingMethod(), 'eabidpdee') === 0;
    }
    
    
    
    /**
     * <p>Converts DPD op=pudo Opening Times into human readable format.</p>
     * <ul>
         <li><b>Input: 1:11:0:16:0,2:7:30:20:0,3:7:30:20:0,4:7:30:20:0,5:7:30:20:0,6:7:30:20:0,7:8:0:16:0</b></li>
         <li><b>Result: (E-R 7:30-20; L 8-16; P 11-16)</b></li>
     </ul>
     * 
     * @param string $dpdOpeningDescription DPD Openings description
     * @param Zend_Locale $locale Locale which is used to print out weekday names
     * @return string
     */
    public function getOpeningsDescriptionFromTerminal($dpdOpeningDescription, $locale = null) {
        $openingTimeFormat = 'H:m';
        $displayTimeFormat = 'H:mm';
        
        if (!$locale) {
            $locale = Mage::app()->getLocale()->getLocaleCode();
        }
        
        //days  start from monday
        $passThruOrder = array('2', '3', '4', '5', '6', '7', '1');
        
        /*
         * Format: array key = weekday name
         */
        $openingDescriptions = array();
        
        //we need these in order to get times in normalized manner
        $startTime = new Zend_Date(0, Zend_Date::TIMESTAMP);
        $endTime = new Zend_Date(0, Zend_Date::TIMESTAMP);
        
        //here are comma separeted opening times
        $openings = explode(',', $dpdOpeningDescription);
        /*
         * Format:
         * <weekday>:<starth>:<startm>:<endh>:<endm>
         * 1=sunday
         * 2=monday
         * ...
         * 7=saturday
         * 
         */
        foreach ($openings as $opening) {
            $openTimePartials = explode(':', $opening);
            $startTime->set($openTimePartials[1].':'.$openTimePartials[2], $openingTimeFormat);
            $endTime->set($openTimePartials[3].':'.$openTimePartials[4], $openingTimeFormat);
            
            if (!isset($openingDescriptions[(string)$openTimePartials[0]])) {
                $openingDescriptions[(string)$openTimePartials[0]] = array();
            }
            $openingDescriptions[(string)$openTimePartials[0]][] = str_replace(':00', '', $startTime->get($displayTimeFormat)) .'-'. str_replace(':00', '', $endTime->get($displayTimeFormat));
            
        }
        
        
        /*
         * Format:
         * array key = day of week digit
         * array value = all opening times for that day separated by comma
         */
        $finalOpeningDescriptions = array();
        $previusOpeningStatement = false;
        $previusWeekdayName = false;
        $firstElement = false;
        
        
        foreach ($passThruOrder as $dayOfWeekDigit) {
            $startTime->set($dayOfWeekDigit - 1, Zend_Date::WEEKDAY_DIGIT);

            $weekDayName = $startTime->get(Zend_Date::WEEKDAY_NARROW, $locale);
            if ($firstElement === false) {
                $firstElement = $previusWeekdayName;
            }
            if (isset($openingDescriptions[$dayOfWeekDigit])) {
                
                $openingStatement = str_replace('0-0', '0-24', implode(',', $openingDescriptions[$dayOfWeekDigit]));
            } else {
                $openingStatement = '';
            }
            
            if ($previusOpeningStatement !== false && $previusOpeningStatement != $openingStatement) {
                //we have a change
                if ($firstElement != $previusWeekdayName) {
                    $finalOpeningDescriptions[] = $firstElement.'-'.$previusWeekdayName.' '.$previusOpeningStatement;
                } else {
                    $finalOpeningDescriptions[] = $previusWeekdayName.' '.$previusOpeningStatement;
                }
                
                
                $firstElement = false;
            }
            $previusOpeningStatement = $openingStatement;
            $previusWeekdayName = $weekDayName;
            
        }
        if ($previusOpeningStatement !== false) {
            if ($previusOpeningStatement !== '') {
                //we have a change
                if (!$firstElement) {
                    $finalOpeningDescriptions[] = $previusWeekdayName . ' ' . $previusOpeningStatement;
                } else {
                    $finalOpeningDescriptions[] = $firstElement . '-' . $previusWeekdayName . ' ' . $previusOpeningStatement;
                }
            }
        }
        
        if (count($finalOpeningDescriptions)) {
            return '('.implode('; ', $finalOpeningDescriptions).')';
        }
        return '';
    }
    

}

