<?php

/*
  
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * <p>Wrapper class for communicating with DPD API</p>
 * <p>Each request is prefilled with username, password, return address data whenever possible.</p>
 * <p>Each response is json_decoded to assoc array and Exception is thrown when response error code is else than integer 0</p>
 *
 * @author Matis
 */
class Eabi_DpdEE_Model_Api extends Varien_Object {
    
    
    
    /**
     * Retrieve information from carrier configuration
     *
     * @param   string $field
     * @return  mixed
     */
    public function getConfigData($field) {
        if (!$this->getCode()) {
            return false;
        }
        $path = 'carriers/'.$this->getCode().'/'.$field;
        return Mage::getStoreConfig($path, $this->getStore());
    }
    
    
    /**
     * <p>Fetches list of parcel terminals from DPD API. (op=pudo)</p>
     * <p>This function can be used without DPD API account.</p>
     * <p>Parcel terminals are included in 'data' array key.</p>
     * @return array
     */
    public function getOfficeList() {
        $body = @$this->_getRequest();
        return $body;
    }
    
    
    /**
     * <p>Fetches available courier collection times. (op=date)</p>
     * @param array $requestData
     * @return array
     */
    public function getCourierCollectionTimes(array $requestData = array()) {
        $details = array(
            'op' => 'date',
            'Po_postal' => $this->getConfigData('return_postcode'),
            'Po_country' => strtolower($this->getConfigData('return_country')),
            'Po_type' => isset($requestData['Po_type'])?strtolower($requestData['Po_type']):'po',
        );
        if ($details['Po_country'] == 'ee') {
            //documentation required 'eesti' as supplied country
            //in order to use same api in other countries, we change it to 'eesti' only for estonia
            //probably in the future all countries require only iso-3166 code
            $details['Po_country'] = 'eesti';
        }
        foreach ($details as $key => $detail) {
            $requestData[$key] = $detail;
        }
        $requestResult = $this->_getRequest($requestData);
        return $requestResult;
        
    }
    
    /**
     * <p>Send parcel data to DPD server, prefills with return data from Magento configuration.</p>
     * @param array $requestData
     * @return array
     */
    public function autoSendData(array $requestData) {
        $returnDetails = array(
            'op' => 'order',
            'Po_name' => $this->getConfigData('return_name'),
            'Po_company' => $this->getConfigData('return_company'),
            'Po_street' => $this->getConfigData('return_street'),
            'Po_postal' => $this->getConfigData('return_postcode'),
            'Po_country' => strtolower($this->getConfigData('return_country')),
            'Po_city' => $this->getConfigData('return_citycounty'),
            'Po_contact' => $this->getConfigData('return_name'),
            'Po_phone' => $this->getConfigData('return_phone'),
            //po-remark
            'Po_email' => $this->getConfigData('return_email'),
            'Po_show_on_label' => $this->getConfigData('po_show_on_label')?'true':'false',
            'Po_save_address' => $this->getConfigData('po_save_address')?'true':'false',
//            'Po_type' => $this->getConfigData('senddata_service'),
            'LabelsPosition' => $this->getConfigData('label_position'),
            'System_name' => 'Magento - ' . Mage::getVersion(),
            
        );
        
        foreach ($returnDetails as $key => $returnDetail) {
            $requestData[$key] = $returnDetail;
        }
        if (!isset($requestData['Po_type'])) {
            $requestData['Po_type'] = $this->getConfigData('senddata_service');
        }
        
        $requestResult = $this->_getRequest($requestData);
        return $requestResult;
    }
    

    /**
     * <p>Determines if courier has been called to pick up the packages.</p>
     * <p>If courier has been called to fetch packages and courier pickup time from has not yet been reached, then it returns array consisting following elements:</p>
     * <ul>
         <li>UNIX timestamp when courier pickup should start</li>
         <li>UNIX timestamp when courier pickup should end</li>
     </ul>
     * <p>On every other scenario this function returns boolean false</p>
     * @return boolean|array
     */
    public function isCourierComing() {
        $pickupTime = $this->getConfigData('courier_pickup_time');
        $time = time();
        if ($pickupTime) {
            $pickupTime = explode(',', $pickupTime);
        }
        if ($pickupTime[0] >= $time) {
            return $pickupTime;
        }
        return false;
    }
    
    
    
    /**
     * <p>Sends actual request to DPD API, prefills with username and password and json decodes the result.</p>
     * <p>Default operation (op=pudo), on such scenario username and password is not sent.</p>
     * <p>If return error code is else than 0, then exception is thrown.</p>
     * @param array $params
     * @param string $url
     * @return array
     */
    protected function _getRequest($params = array('op' => 'pudo'), $url = null) {
        if (!$url) {
            $url = $this->getConfigData('api_url');
        }
        if (isset($params['op']) && $params['op'] != 'pudo') {
            $params['User_login'] = $this->getConfigData('sendpackage_username');
            $params['User_password'] = $this->getConfigData('sendpackage_password');
        }
        $client = new Zend_Http_Client($url);
        $options = array(
            'timeout' => $this->getConfigData('http_request_timeout') > 10 ? $this->getConfigData('http_request_timeout') : 10,
        );
        $client->setConfig($options);
        $dataToLog = array(
            'url' => $url,
            'method' => Zend_Http_Client::POST,
            'params' => $this->_debugReplace($params),
        );
        try {
            $client->setParameterPost($params);
            $resp = $client->request(Zend_Http_Client::POST);
        } catch (Exception $ex) {
            $dataToLog['exception'] = $ex->__toString();
            if ($this->_isLogEnabled()) {
                Mage::log($dataToLog, Zend_Log::DEBUG, __CLASS__ . '.log');
            }
            throw $ex;
        }


        $decodeResult = @json_decode($resp->getBody(), true);
        if (!is_array($decodeResult) || !isset($decodeResult['Error_code'])
                || $decodeResult['Error_code'] !== 0) {
            if (!$decodeResult) {
                $dataToLog['response'] = $resp;
            } else {
                $dataToLog['response'] = $decodeResult;
            }
            if ($this->_isLogEnabled()) {
                Mage::log($dataToLog, Zend_Log::ERR, __CLASS__ . '.log');
            }
            
            Mage::throwException($this->_getDpdHelper()->__('DPD request failed with response: %s', print_r($decodeResult, true)));
        }
        $dataToLog['response'] = $decodeResult;
//        $dataToLog['client'] = $client;
        if ($this->_isLogEnabled()) {
            Mage::log($dataToLog, Zend_Log::DEBUG, __CLASS__ . '.log');
        }
        
        return $decodeResult;
    }
    
    
    protected function _debugReplace($params) {
        $res = $params;
        $toReplace = array(
            'User_login',
            'User_password',
        );
        foreach ($toReplace as $replace) {
            if (isset($res[$replace])) {
                $res[$replace] = '***';
            }
        }
        return $res;
    }
    
    
    protected function _isLogEnabled() {
        return $this->getConfigData('enable_log');
    }
    
    /**
     * 
     * @return Eabi_DpdEE_Helper_Data
     */
    protected function _getDpdHelper() {
        return Mage::helper('eabi_dpdee');
    }
    
    
    
    /**
     * 
     * @return Eabi_Postoffice_Helper_Data
     */
    protected function _getOfficeHelper() {
        return Mage::helper('eabi_postoffice');
    }
    
    
}
