<?php

/*
   *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Livehandler
 * @copyright  Copyright (c) 2018 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * <p>Helper class for generating RSA public and private keys</p>
 *
 * @author Matis
 */
class Eabi_Livehandler_Helper_Keypair extends Mage_Core_Helper_Abstract {
    
    
    /**
     * <p>Generates new RSA keypair and returns it as assoc array with following keys:</p>
     * <ul>
         <li><strong>privkey</strong> - private key .pem encoded string</li>
         <li><strong>csr</strong> - public key which has been signed with generated private key</li>
         <li><strong>pubkey</strong> - public key .pem encoded certificate string</li>
     </ul>
     * <p>Input uses following options:</p>
     * <ul>
         <li><strong>countryName</strong> - Country code uppercase goes here, like EE</li>
         <li><strong>stateOrProvinceName</strong> - County or state name goes here</li>
         <li><strong>localityName</strong> - Name of the city goes here</li>
         <li><strong>organizationName</strong> - Name of the company goes here</li>
         <li><strong>organizationalUnitName</strong> - Company www goes here</li>
         <li><strong>commonName</strong> - Name of the person who generated this cert goes here or name of the person on board</li>
         <li><strong>emailAddress</strong> - Ggeneric company e-mail goes here</li>
     </ul>
     * <p>Private key is not encrypted with any password</p>
     * 
     * @param array $input
     * @param array $conf
     */
    public function generateKeyPair(array $input, array $conf = array('private_key_bits' => 2048), $numberofdays = 3650) {
        $publickey = null;
        $privatekey = null;
        $csrStr = null;
	$privkeypass = null;
	$privkey = openssl_pkey_new($conf);
	$csr = openssl_csr_new($input, $privkey);
	$sscert = openssl_csr_sign($csr, null, $privkey, $numberofdays);
	openssl_x509_export($sscert, $publickey);
	openssl_pkey_export($privkey, $privatekey, $privkeypass);
	openssl_csr_export($csr, $csrStr);
	$pubkey=openssl_pkey_get_details($privkey);
	$pubkey=$pubkey["key"];
        
        return array(
            'privkey' => $privatekey,
            'pubkey' => $publickey,
            'csr' => $csrStr,
        );
    }

    /**
     * <p>Returns assoc array of values used to initiate public-private certificates for this installation.</p>
     * <p>Default values are:</p>
     * <ul>
         <li><strong>countryName</strong> - Default declared country for this installation</li>
         <li><strong>stateOrProvinceName</strong> - N/A</li>
         <li><strong>localityName</strong> - N/A</li>
         <li><strong>organizationName</strong> - N/A</li>
         <li><strong>organizationalUnitName</strong> - Store base url hosts, comma separated</li>
         <li><strong>commonName</strong> - Aktsiamaailm LLC</li>
         <li><strong>emailAddress</strong> - Default general e-mail address for this installation</li>
     </ul>
     * @return array
     */
    final public function getDefaultKeyData() {
        $keyInputData = array(
            "countryName" => $this->_getStringOrDefault($this->_getStoreConfig('general/country/default'), 'US'),
            "stateOrProvinceName" => 'N/A',
            "localityName" => 'N/A',
            "organizationName" => 'N/A',
            "organizationalUnitName" => $this->_getStringOrDefault(implode(',', $this->_getEabi()->getAllStoreUrls())),
            "commonName" => 'Aktsiamaailm LLC',
            "emailAddress" => $this->_getStringOrDefault($this->_getStoreConfig('trans_email/ident_general/email')),
        );
        
        return $keyInputData;
    }
    
    /**
     * <p>Returns <code>input</code> or <code>default</code> if input is empty of false or not string</p>
     * @param string $input
     * @param string $default
     * @return string
     */
    private function _getStringOrDefault($input, $default = 'N/A') {
        if (!$input || !is_string($input)) {
            return (string)$default;
        }
        return $input;
    }


    /**
     * <p>Fetches config data when Mage::getStoreConfig is not available</p>
     * @param string $path
     * @return boolean|string
     */
    protected function _getStoreConfig($path) {
        $storeId = Mage_Core_Model_App::ADMIN_STORE_ID;
        $configItem = $this->_getCoreConfigDataModel()
                ->getCollection()
                ->addFieldToFilter('path', $path)
                ->addFieldToFilter('scope_id', Mage_Core_Model_App::ADMIN_STORE_ID)
            ->getFirstItem();
        if ($configItem) {
            return $configItem->getValue();
        }
        return false;
    }
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabi() {
        return Mage::helper('eabi');
    }
    
    /**
     * 
     * @return Mage_Core_Model_Config_Data
     */
    protected function _getCoreConfigDataModel() {
        return Mage::getModel('core/config_data');
    }
    

}
