<?php

/*
   *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Livehandler
 * @copyright  Copyright (c) 2018 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * <p>Lightweight Logger class which logs all queries to getLogFile() file.</p>
 * <p>Initialization example:</p>
 * <pre>
        $logger = $this-&gt;getEabiHelper()
        &nbsp;&nbsp;&nbsp;&nbsp;-&gt;getEabiLogger();
        $logger-&gt;setLogFile('name-of-log-file-'.date('Y-m-d').'.log')
        &nbsp;&nbsp;&nbsp;&nbsp;-&gt;setLogLevel(Zend_Log::DEBUG)
        &nbsp;&nbsp;&nbsp;&nbsp;-&gt;setIsLogEnabled(true)
        &nbsp;&nbsp;&nbsp;&nbsp;-&gt;setLogPrefix('log prefix');
        return $logger;
 * </pre>
 * <p>See also source model of <code>eabi_livehandler/source_log</code> which refers to class <code>Eabi_Livehandler_Model_Source_Log</code></p>
 * 
 *
 * @author Matis
 */
class Eabi_Livehandler_Model_Logger {

    private static $_isRequestLogged = false;
    
    /**
     *
     * @var string 
     */
    protected $logPrefix = '';
    
    /**
     *
     * @var string 
     */
    protected $logFile = 'eabi-livehandler.log';
    
    /**
     *
     * @var int 
     */
    protected $logLevel = Zend_Log::DEBUG;
    
    /**
     *
     * @var bool 
     */
    protected $isLogEnabled = false;
    
    
    protected $sensitiveVariableNames = array();

    /**
     * <p>Gets logging prefix for current logger instance</p>
     * @return string
     */
    public function getLogPrefix() {
        return $this->logPrefix;
    }

    /**
     * <p>Gets log file name for current logger instance</p>
     * @return string
     */
    public function getLogFile() {
        return $this->logFile;
    }

    
    /**
     * <p>Gets logging level for current logger instance</p>
     * @return int
     */
    public function getLogLevel() {
        return $this->logLevel;
    }

    /**
     * <p>Gets is log enabled flag for current logger instance</p>
     * @return bool
     */
    public function getIsLogEnabled() {
        return (bool)$this->isLogEnabled;
    }


    /**
     * <p>Sets logger prefix for current logger instance</p>
     * @param string $logPrefix
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function setLogPrefix($logPrefix) {
        $this->logPrefix = $logPrefix;
        return $this;
    }

    /**
     * <p>Sets log file name for current logger instance</p>
     * <p>Example: 'my-log.log'</p>
     * <p>Gets logged to ~/var/log/my-log.log</p>
     * @param string $logFile
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function setLogFile($logFile) {
        $this->logFile = $logFile;
        return $this;
    }

    /**
     * <p>Sets logging level for current logger instance</p>
     * <p>Allowed values are (range emerg=0, debug=7):</p>
     * <ul>
         <li>Zend_Log::DEBUG</li>
         <li>Zend_Log::INFO</li>
         <li>Zend_Log::NOTICE</li>
         <li>Zend_Log::WARN</li>
         <li>Zend_Log::ERR</li>
         <li>Zend_Log::CRIT</li>
         <li>Zend_Log::ALERT</li>
         <li>Zend_Log::EMERG</li>
     </ul>
     * @param int $logLevel
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function setLogLevel($logLevel) {
        $this->logLevel = $logLevel;
        return $this;
    }

    /**
     * <p>Enables logging for current logger instance</p>
     * @param bool $isLogEnabled
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function setIsLogEnabled($isLogEnabled) {
        $this->isLogEnabled = (bool)$isLogEnabled;
        return $this;
    }

    /**
     * <p>Returns names of sensitive variables for current logger instance</p>
     * @return array
     */
    public function getSensitiveVariableNames() {
        return $this->sensitiveVariableNames;
    }

    /**
     * <p>Sets array of sensitive variable names, which will be replaced with <code>***</code> when they are contained in <code>$_GET</code> or <code>$_POST</code></p>
     * @param array $sensitiveVariableNames
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function setSensitiveVariableNames(array $sensitiveVariableNames) {
        $this->sensitiveVariableNames = $sensitiveVariableNames;
        return $this;
    }

    
    /**
     * <p>Makes actual logging attempt to <code>$logFile</code></p>
     * <p><code>$level</code> is not checked, and is written to log file as log information</p>
     * <p><code>$_POST</code> and <code>$_GET</code> and <code>User Agent</code> are logged once per request</p>
     * <p><code>Exception</code> types are casted into strings for better readability</p>
     * @param mixed $data data to be writted to logfile
     * @param string $logPrefix prefix for the data to be written
     * @param string $logFile name of the file to write log into
     * @param int $level loggin level for current entry
     * @param array $sensitiveVariables array of sensitive variables to be removed from GET or POST
     */
    protected static function _log($data, $logPrefix, $logFile, $level = Zend_Log::DEBUG, array $sensitiveVariables = array()) {
        $postLevel = Zend_Log::DEBUG;

        if (!self::$_isRequestLogged) {
            if (isset($_POST) && count($_POST)) {
                Mage::log('POST=' . print_r(self::_removeSensitiveVariables($_POST, $logPrefix, $sensitiveVariables), true), $postLevel, $logFile);
            }
            if (isset($_GET) && count($_GET)) {
                Mage::log('GET=' . print_r(self::_removeSensitiveVariables($_GET, $logPrefix, $sensitiveVariables), true), $postLevel, $logFile);
            }
            if (isset($_SERVER) && isset($_SERVER['HTTP_USER_AGENT'])) {
                Mage::log('USER_AGENT=' . $_SERVER['HTTP_USER_AGENT'], $postLevel, $logFile);
            }
            self::$_isRequestLogged = true;
        }

        if ($data instanceof Exception) {
            $data = $data->__toString();
        }
        Mage::log(sprintf(' %s ', $logPrefix) . ': ' . print_r($data, true), $level, $logFile, true);
    }

    /**
     * <p>If <code>$input</code> is array and it contains sensitive variables, then those will be replaced by <code>***</code> symbols.</p>
     * @param mixed $input
     * @param string $logPrefix
     * @return mixed
     */
    protected static function _removeSensitiveVariables($input, $logPrefix, $sensitiveVariables) {
        $result = $input;
        if (!is_array($result)) {
            return $result;
        }
        $sensitives = $sensitiveVariables;
        foreach ($sensitives as $sensitive) {
            if (isset($result[$sensitive])) {
                $result[$sensitive] = '***';
            }
        }
        
        return $result;
    }

    /**
     * <p>Writes <code>$dataToLog</code> into specified log file, if <code>$this->getIsLogEnabled()</code> is set to true and <code>$this->getLogLevel()</code> is equal or greated than <code>$level</code></p>
     * <p>If <code>$dataToLog</code> is instance of <code>Exception</code> then <code>$level</code> is automatically set to <code>Zend_Log::ERR</code> (3)</p>
     * @param mixed $dataToLog
     * @param int $level
     * @return \Eabi_Livehandler_Model_Logger
     */
    protected function log($dataToLog, $level = Zend_Log::DEBUG) {
        $file = $this->getLogFile();
        if (!$file) {
            $file = 'eabi-livehandler.log';
        }
        if (is_object($dataToLog) && $dataToLog instanceof Exception) {
            $level = Zend_Log::ERR;
        }

        if ($this->getIsLogEnabled() && $this->getLogLevel() >= $level) {
            self::_log($dataToLog, $this->getLogPrefix(), $file, $level, $this->getSensitiveVariableNames());
        }
        return $this;
    }
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::ERR</code> (3)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function error($dataToLog) {
        return $this->log($dataToLog, Zend_Log::ERR);
    }
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::ERR</code> (3)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function debug($dataToLog) {
        return $this->log($dataToLog, Zend_Log::DEBUG);
    }
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::INFO</code> (6)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function info($dataToLog) {
        return $this->log($dataToLog, Zend_Log::INFO);
    }
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::ALERT</code> (1)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function alert($dataToLog) {
        return $this->log($dataToLog, Zend_Log::ALERT);
    }
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::CRIT</code> (2)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function crit($dataToLog) {
        return $this->log($dataToLog, Zend_Log::CRIT);
    }
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::EMERG</code> (0)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function emerg($dataToLog) {
        return $this->log($dataToLog, Zend_Log::EMERG);
    }
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::NOTICE</code> (5)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function notice($dataToLog) {
        return $this->log($dataToLog, Zend_Log::NOTICE);
    }
    
    
    
    /**
     * <p>Writes <code>$dataToLog</code> into current logger instance with level of <code>Zend_Log::WARN</code> (4)</p>
     * @param mixed $dataToLog data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function warn($dataToLog) {
        return $this->log($dataToLog, Zend_Log::WARN);
    }
    
    /**
     * <p>Writes current stackTrace into current logger instance, which can be prefixed with optional <code>$extraInfo</code></p>
     * @param mixed $extraInfo data to be logged
     * @return \Eabi_Livehandler_Model_Logger
     */
    public function logStackTrace($extraInfo = null) {
        $dataToLog = array();
        $dataToLog['info'] = $extraInfo;
        $stack = debug_backtrace();
        $dataToLog['stack'] = '';
        foreach ($stack as $key => $info) {
            $dataToLog['stack'] .= "#" . $key . " Called " . $info['function'] . " in " . $info['file'] . " on line " . $info['line'] . "\r\n";
        }

        return $this->log($dataToLog, Zend_Log::DEBUG);
    }
    
    
}
